from typing import List
from .client_functions import get, post, delete
from .pagination import pageQuery, PaginatedResponse
from .exceptions import SignalBoxException


def create_parameterset_recommender(access_token: str, base_url: str, common_id: str, name: str, parameters: List[str], bounds, arguments):
    json_params = {
        "commonId": common_id,
        "name": name,
        "parameters": parameters,
        "bounds": bounds,
        "arguments": arguments,
    }
    r = post(f'{base_url}/api/recommenders/ParameterSetRecommenders',
             json_params, access_token)
    if r.ok:
        return r.json()
    else:
        raise SignalBoxException(r.text)


def get_parameterset_recommender(access_token: str, base_url: str, common_id: str):
    r = get(
        f'{base_url}/api/recommenders/ParameterSetRecommenders/{common_id}', access_token)
    if r.ok:
        return r.json()
    else:
        raise SignalBoxException(r.text)


def delete_parameterset_recommender(access_token: str, base_url: str, id):
    r = delete(
        f'{base_url}/api/recommenders/ParameterSetRecommenders/{id}', access_token)
    if r.ok:
        return r.json()
    else:
        raise SignalBoxException(r.text)



from .client_functions import get, post
from .pagination import pageQuery, PaginatedResponse
from .exceptions import SignalBoxException


def create_feature(access_token: str, base_url: str, common_id: str, name: str):
    json_params = {
        "commonId": common_id,
        "name": name,
    }
    r = post(f'{base_url}/api/features',
             json_params, access_token)
    if r.ok:
        return r.json()
    else:
        raise SignalBoxException(r.text)


def get_feature(access_token: str, base_url: str, id):
    r = get(f'{base_url}/api/features/{id}', access_token)
    if r.ok:
        return r.json()
    else:
        raise SignalBoxException(r.text)


def set_feature_value(access_token: str, base_url: str, user_id, feature_id: str, value):
    json_params = {
        "value": value,
    }
    r = post(f'{base_url}/api/trackedusers/{user_id}/features/{feature_id}',
             json_params, access_token)
    if r.ok:
        return r.json()
    else:
        raise SignalBoxException(r.text)

def get_feature_value(access_token: str, base_url: str, user_id, feature_id: str, version: int):
    version_qs = f'?version={version}' if version is not None else ""
    r = get(f'{base_url}/api/trackedusers/{user_id}/features/{feature_id}{version_qs}', access_token)
    if r.ok:
        return r.json()
    else:
        raise SignalBoxException(r.text)



from typing import List
from .client_functions import get, post, delete
from .exceptions import SignalBoxException
from .target_variables import create_recommender_target_variable_value, get_recommender_target_variable_values


def create_product_recommender(access_token: str, base_url: str, common_id: str, name: str, product_ids: List[str] = None):
    json_params = {
        "commonId": common_id,
        "name": name,
        "productIds": product_ids
    }
    r = post(f'{base_url}/api/recommenders/ProductRecommenders/',
             json_params, access_token)
    if r.ok:
        return r.json()
    else:
        raise SignalBoxException(r.text)


def get_product_recommender(access_token: str, base_url: str, id):
    r = get(f'{base_url}/api/recommenders/ProductRecommenders/{id}', access_token)
    if r.ok:
        return r.json()
    else:
        raise SignalBoxException(r.text)


def invoke_product_recommender(access_token: str, base_url: str, id: int, common_user_id: str):
    json_params = {
        "commonUserId": common_user_id
    }
    r = post(f'{base_url}/api/recommenders/ProductRecommenders/{id}/invoke',
             json_params, access_token)
    if r.ok:
        return r.json()
    else:
        raise SignalBoxException(r.text)


def get_parameterset_recommender(access_token: str, base_url: str, id):
    r = delete(
        f'{base_url}/api/recommenders/ProductRecommenders/{id}', access_token)
    if r.ok:
        return r.json()
    else:
        raise SignalBoxException(r.text)


def delete_product_recommender(access_token: str, base_url: str, id):
    r = delete(
        f'{base_url}/api/recommenders/ProductRecommenders/{id}', access_token)
    if r.ok:
        return r.json()
    else:
        raise SignalBoxException(r.text)


def create_target_variable_value(access_token: str, base_url: str, id: int, start, end, name: str, value: float):
    return create_recommender_target_variable_value(access_token, base_url, "ProductRecommenders", id, start, end, name, value)


def get_target_variable_values(access_token: str, base_url: str, id: int,  name: str = None):
    return get_recommender_target_variable_values(access_token, base_url, "ProductRecommenders", id, name)



from .client_functions import get, post
from .exceptions import SignalBoxException


def create_recommender_target_variable_value(access_token: str, base_url: str, recommender_api_name: str, id: int, start, end, name: str, value: float):
    json_params = {
        "start": start,
        "end": end,
        "name": name,
        "value": value,
    }
    r = post(f'{base_url}/api/recommenders/{recommender_api_name}/{id}/TargetVariableValues',
             json_params, access_token)
    if r.ok:
        return r.json()
    else:
        raise SignalBoxException(r.text)


def get_recommender_target_variable_values(access_token: str, base_url: str, recommender_api_name: str, id: int, name: str = None):

    r = get(f'{base_url}/api/recommenders/{recommender_api_name}/{id}/TargetVariableValues?name={name if name is not None else ""}', access_token)
    if r.ok:
        return r.json()
    else:
        raise SignalBoxException(r.text)



from .client_functions import get, post
from .pagination import pageQuery, PaginatedResponse
from .exceptions import SignalBoxException


def create_product(access_token: str, base_url: str, common_id: str, name: str, list_price: float, direct_cost: float, description: str):
    json_params = {
        "commonId": common_id,
        "name": name,
        "listPrice": list_price,
        "directCost": direct_cost,
        "description": description,
    }
    r = post(f'{base_url}/api/products',
             json_params, access_token)
    if r.ok:
        return r.json()
    else:
        raise SignalBoxException(r.text)


def query_products(access_token: str, base_url: str, page: int = None):
    r = get(f'{base_url}/api/products?{pageQuery(page)}', access_token)
    if r.ok:
        return PaginatedResponse(**r.json())
    else:
        raise SignalBoxException(r.text)



from typing import List
from .client_functions import get, post, delete
from .exceptions import SignalBoxException


def create_items_recommender(access_token: str, base_url: str, common_id: str, name: str, default_item_id: str, item_ids: List[str] = None, number_items_to_recommend: int = None):
    json_params = {
        "commonId": common_id,
        "name": name,
        "itemIds": item_ids,
        "defaultItemId": default_item_id,
        "numberOfItemsToRecommend": number_items_to_recommend
    }
    r = post(f'{base_url}/api/recommenders/ItemsRecommenders/',
             json_params, access_token)
    if r.ok:
        return r.json()
    else:
        raise SignalBoxException(r.text)


def get_items_recommender(access_token: str, base_url: str, id):
    r = get(f'{base_url}/api/recommenders/ItemsRecommenders/{id}', access_token)
    if r.ok:
        return r.json()
    else:
        raise SignalBoxException(r.text)


def invoke_items_recommender(access_token: str, base_url: str, id: int, common_user_id: str, arguments: dict = None):
    json_params = {
        "commonUserId": common_user_id,
        "arguments": arguments
    }
    r = post(f'{base_url}/api/recommenders/ItemsRecommenders/{id}/invoke',
             json_params, access_token)
    if r.ok:
        return r.json()
    else:
        raise SignalBoxException(r.text)


def delete_items_recommender(access_token: str, base_url: str, id):
    r = delete(
        f'{base_url}/api/recommenders/ItemsRecommenders/{id}', access_token)
    if r.ok:
        return r.json()
    else:
        raise SignalBoxException(r.text)



from context import signalboxclient
# import client
import datetime
import random
import uuid
import numpy as np
import environment

creds = signalboxclient.client.Credentials(environment.api_key)
test_client = signalboxclient.client.SignalBoxClient(creds)

event_types = ["CREATED", "CANCELED", "STARTED"]
event_kinds = ["BILLING", "TICKET", "PAYMENT"]

my_token = test_client.access_token
# create user
user_id = str(uuid.uuid1())
u = test_client.create_user('testuser', user_id)
print('created user with id=', user_id)
# update the user
u['properties'] = {
    'client_test.py': datetime.datetime.now().strftime('%Y-%m-%dT%H:%M:%S.%f%z')
}
test_client.create_or_update_users([u])
offer = test_client.create_offer('my_offer', 'AUD', 100, 50)
offer_id = offer['id']

# create a touchpoint
touchpoint_id = "client_test.py"
tp = test_client.create_touchpoint_on_tracked_user(
    user_id, touchpoint_id, {"key": "value"})

assert tp['version'] > 0
touchpoint_id = "client_test.py"
tp = test_client.get_touchpoint_on_tracked_user(
    user_id, touchpoint_id)

assert tp["values"]["key"] == "value"
# get the touchpoint
touchpoint = test_client.get_touchpoint(touchpoint_id)
assert touchpoint['commonId'] == touchpoint_id

# create a product
product_id = str(uuid.uuid1())
product = test_client.create_product(
    product_id, "client_test.py Product", 10, None, "A test description.")
assert product['id'] > 0

# # query the products
response = test_client.query_products(page=1)
assert len(response.items) > 1

# create a product recommender
product_recommender_id = str(uuid.uuid1())
product_recommender = test_client.create_product_recommender(
    product_recommender_id, "client_test.py Recommender")
assert product_recommender['id'] is not None

for i in range(1, 10):
    start = datetime.datetime(2020, i, 1).strftime('%Y-%m-%dT%H:%M:%S.%f%z')
    end = datetime.datetime(2020, i+1, 1).strftime('%Y-%m-%dT%H:%M:%S.%f%z')
    test_client.create_product_recommender_target_variable_value(
        product_recommender['id'], start, end, "LTV", i * 7 * random.randint(1, 3))

target_variable_values = test_client.get_product_recommender_target_variable_values(
    product_recommender_id)
assert len(target_variable_values) > 0

# create some events with revenue associated using the recommendation correlator id
events = []
for i in range(100):
    uid = str(uuid.uuid1())
    recommendation = test_client.invoke_product_recommender(
        product_recommender['id'], uid)
    print(f'Event User Id: ', uid)
    properties = {}
    e_type = np.random.choice(event_types)
    e_kind = np.random.choice(event_kinds)
    if(e_kind == "PAYMENT"):
        properties['revenue'] = random.random() * 100
    events.append(test_client.construct_event(
        uid, str(uuid.uuid1()), e_type, e_kind, properties=properties,
        recommendationCorrelatorId=recommendation["correlatorId"]))

print(f'logging {len(events)} events')
result = test_client.log_events(events)
print(result)

# delete the recommender
delete_response = test_client.delete_product_recommender(
    product_recommender['id'])
assert delete_response['id'] is not None


# create a parameter
parameter_id = str(uuid.uuid1())
parameter = test_client.create_parameter(
    common_id=parameter_id, name="client_test.py", default_value=5)
assert parameter['id'] is not None
assert parameter['commonId'] == parameter_id
print("Created Parameter")

# create a feature
feat_id = f'client.py-{uuid.uuid1()}'
feat = test_client.create_feature(feat_id, "Python Client Feature")
feat_r = test_client.get_feature(feat_id)
assert feat_r['id'] == feat['id']
# check we can set a feature and get a feature on a user
test_client.set_feature_value(user_id, feat_id, 5)
feature_value = test_client.get_feature_value(user_id, feat_id)
assert feature_value['value'] == 5
print('tracked user with feature value:', user_id)

events = []
for i in range(100):
    uid = str(uuid.uuid1())
    print(f'Event User Id: ', uid)
    properties = {}
    e_type = np.random.choice(event_types)
    e_kind = np.random.choice(event_kinds)
    if(e_kind == "PAYMENT"):
        properties['amountBilled'] = random.random() * 100
    events.append(test_client.construct_event(
        uid, str(uuid.uuid1()), e_type, e_kind, properties=properties))
    # commonUserId, event_id, event_type, kind, properties, timestamp, source_system_id

print(f'logging {len(events)} events')
result = test_client.log_events(events)
print(result)

print('\ntests successful')




from .client_functions import get, post
from .pagination import pageQuery, PaginatedResponse
from .exceptions import SignalBoxException

def create_parameter(access_token: str, base_url: str, common_id: str, name: str, parameter_type: str, default_value, description: str):
    json_params = {
        "commonId": common_id,
        "name": name,
        "parameterType": parameter_type,
        "description": description,
        "defaultValue": default_value
    }
    r = post(f'{base_url}/api/parameters',
             json_params, access_token)
    if r.ok:
        return r.json()
    else:
        raise SignalBoxException(r.text)


#!/usr/bin/env python
# setup.py generated by flit for tools that don't yet use PEP 517

from distutils.core import setup

packages = \
['signalboxclient']

package_data = \
{'': ['*']}

setup(name='signalboxclient',
      version='0.1.21',
      description='A library used as a Four2/SignalBox client.',
      author='Rian Finnegan',
      author_email='rian@four2.ai',
      url='https://www.four2.ai',
      packages=packages,
      package_data=package_data,
     )
